@@ -3,6 +3,7 @@ from __future__ import (absolute_import, division, print_function,
 from algae.functions import identity
 from functools import partial
 from operator import lt
+from weakref import ref
 
 class RedBlackTree(object):
     unspecified = object()
@@ -405,16 +406,6 @@ the node must not be None.
             x.red = False
         return
 
-    def __rb_min(self):
-        """Returns the minimum node of the tree."""
-        if self.root is None:
-            return None
-
-        n = self.root
-        while n.left is not None:
-            n = n.left
-        return n
-
     def __getitem__(self, key):
         node = self.find_node(key)
         if node is None:
@@ -528,16 +519,35 @@ def generate_nodes(node, transform=identity):
             yield child
 
 class RedBlackTreeNode(object):
+    __slots__ = ["red", "parentref", "left", "right", "key", "value",
+                 "__weakref__"]
+
     def __init__(self, key, value):
         super(RedBlackTreeNode, self).__init__()
         self.red = True
-        self.parent = None
+        self.parentref = None
         self.left = None
         self.right = None
         self.key = key
         self.value = value
         return
 
+    @property
+    def parent(self):
+        if self.parentref is None:
+            return None
+        else:
+            return self.parentref()
+    
+    @parent.setter
+    def parent(self, value):
+        if value is None:
+            self.parentref = None
+        else:
+            self.parentref = ref(value)
+
+        return
+
     def debug(self):
         if self.left is not None:
             left = self.left.debug().split("\n")
@@ -599,8 +609,8 @@ class RedBlackTreeNode(object):
                 right_height += 1
 
         if left_height != right_height:
-            raise ValueError("left and right black heights are unequal: "
-                             "%d vs %d" % (left_height, right_height))
+            raise AssertionError("left and right black heights are unequal: "
+                                 "%d vs %d" % (left_height, right_height))
 
         return left_height
 
